#include "fixmath.h"

inline uint32_t fix_fract(uint32_t inNumerator, uint32_t inDenominator) {
	if(inDenominator <= inNumerator)
		return 0xFFFFFFFF;
	uint32_t tempMod = (inNumerator % inDenominator);
	uint32_t tempDiv = (0xFFFFFFFF / (inDenominator - 1));
	return (tempMod * tempDiv);
}

inline uint32_t fix_fract_invert(uint32_t inFract) {
	return (0xFFFFFFFF - inFract);
}

inline uint32_t fix_fract_usmul(uint32_t inVal, uint32_t inFract) {
	return (uint32_t)(((uint64_t)inVal * (uint64_t)inFract) >> 32);
}

inline uint32_t fix_fract_smul(int32_t inVal, uint32_t inFract) {
	if(inVal < 0)
		return -fix_fract_usmul(-inVal, inFract);
	return fix_fract_usmul(inVal, inFract);
}



inline uint32_t fix_usmul(uint32_t inArg0, uint32_t inArg1, uint32_t inShift) {
	uint64_t tempResult = ((uint64_t)inArg0 * (uint64_t)inArg1) + (1 << (inShift - 1));
	return (tempResult >> inShift);
}

inline int32_t fix_smul(int32_t inArg0, int32_t inArg1, uint32_t inShift) {
	int64_t tempResult = ((int64_t)inArg0 * (int64_t)inArg1) + (1 << (inShift - 1));
	return (tempResult >> inShift);
}



inline uint32_t fix_usdiv(uint32_t inArg0, uint32_t inArg1, uint32_t inShift) {
	uint64_t tempOut = inArg0;
	tempOut <<= inShift;
	tempOut += (inArg1 >> 1);
	tempOut /= inArg1;
	return tempOut;
}

inline int32_t fix_sdiv(int32_t inArg0, int32_t inArg1, uint32_t inShift) {
	int64_t tempOut = inArg0;
	tempOut <<= inShift;
	tempOut += (inArg1 >> 1);
	tempOut /= inArg1;
	return tempOut;
}

inline uint32_t fix_ussatadd(uint32_t inArg0, uint32_t inArg1) {
	if((0 - inArg1) >= inArg0)
		return 0xFFFFFFFF;
	return (inArg0 + inArg1);
}

inline int32_t fix_ssatadd(int32_t inArg0, int32_t inArg1) {
	int32_t tempResult = (inArg0 + inArg1);
	if((tempResult > 0) && (inArg0 < 0) && (inArg1 < 0))
		return 0x80000000;
	if((tempResult < 0) && (inArg0 > 0) && (inArg1 > 0))
		return 0x7FFFFFFF;
	return tempResult;
}
